@@ -42,8 +42,8 @@ daylength<-function(latitude,JDay,notimes.as.na=FALSE)
   if(latitude>90|latitude<(-90)) warning("'latitude' is usually between -90 and 90")
 
   Gamma<-2*pi/365*((JDay)-1)
-  Delta<-180/pi*(0.006918-0.399912*cos(Gamma)+0.070257*sin(Gamma)-0.006758*cos(Gamma)+
-                   0.000907*sin(Gamma)-0.002697*cos(3*(Gamma))+0.00148*sin(3*(Gamma)))
+  Delta<-180/pi*(0.006918-0.399912*cos(Gamma)+0.070257*sin(Gamma)-0.006758*cos(2*Gamma)+
+                   0.000907*sin(2*(Gamma))-0.002697*cos(3*(Gamma))+0.00148*sin(3*(Gamma)))
   CosWo<-(sin(-0.8333/360*2*pi)-sin(latitude/360*2*pi)*
             sin(Delta/360*2*pi))/(cos(latitude/360*2*pi)*cos(Delta/360*2*pi))
   
